

#include "processor.inc"
#include "autop_defines.inc"

;************************************************************************************
;																					*
;	The module for servicing the PWM timer 2 interrupts to generate servo pulses.	*
;																					*
;	The servos require one pulse approximately every 20 milliseconds.				*
;	They cannot be sent much more often than that to the common "analog" servos,	*
;	because the "pulse stretchers" would wind up causing much trouble, including	*
;	short circuiting the power supply. The newer "digital" servos would have been	*
;	easier, but this works well enough.												*
;																					*
;	The position of the servo is determined by the pulse width.						*
;	A 1 millisecond pulse is "hard left", a 2 millisecond pulse is "hard right",	*
;	and "straight ahead" is about 1.5 milliseconds.									*
;																					*
;	So, our job is to create 1 to 2 millisecond pulses every 20 milliseconds.		*
;	The PWM timer 2 is set up to generate an interrupt about every millisecond.		*
;	Most of the time, the interrupt handler simply counts and skips pulses.			*
;	When it is time to send a pulse to the servo, the software creates two pulses	*
;	back to back. The first pulse is a full 1 millisecond pulse. The second pulse	*
;	is a partial pulse, from 0 to 1 milliseconds. The servo sees one pulse, from	*
;	1 to 2 milliseconds long. The pulse width is controlled by the "duty cycle",	*
;	(PWM_dc) such that the pulse width in milliseconds is equal to 1 plus 			*
;	PWM_dc/256.																		*
;																					*
;	This module is also a convenient place to query the compass.					*
;	The best the compass can do is one conversion every 33 milliseconds.			*
;	For our purposes, once every 100 milliseconds is fast enough, so the compass	*
;	is read once every 5th servo pulse.												*
;																					*
;************************************************************************************



	EXTERN	get_bearing, Steer
;
;	get_bearing reads the compass, Steer decides where to point the front wheels.
;

PROG1 	CODE

;	Initialization code:

PWMInit

	movlw	0xFE		; maximum pulse width minus 1 to make sure there is no gap between pulses.
	movwf	PR2			;initialize PWM period 
	movlw	0x80		;initialize PWM duty cycle for "straight ahead"
	movwf	PWM1_dc
	movwf	CCPR1L
	movwf	PWM2_dc
	movwf	CCPR2L
	clrf	CCP1CON
	clrf	CCP2CON
;
;	Timer 2 is driven off the crystal clock, which is running at 4X4Mhz=16Mhz.
;	Timer 2 sees 1/4 of the crystal clock, or 4Mhz.
;	The prescaler of 16 divides this down to 250Khz, for a small time tic of 4 microseconds.
;	It takes 255 small tics to generate an interrupt, or about once every millisecond:

	movlw	0x03			;postscale 1:1, prescaler 16, Timer2 OFF
	movwf	T2CON

;	in checking the period of radio PWM, I made a mistake and took decimal instead of hex

	movlw	.22			; 22 milliseconds for the truck is the right number
	movwf	PWM_count
	movwf	PWM_skip

	movlw	0x05		; read compass every 5 pulses, or 10 times per second
	movwf	CMP_count
	movwf	CMP_skip

	return

;
;	service the PWM interrupt once every millisecond
serv_PWM
	bcf		PIR1, TMR2IF		; turn off the interrupt flag
	dcfsnz	PWM_count , F		; decrement the pulse count
	movff	PWM_skip , PWM_count	; reload the skip counter
	movlw	0x8
	subwf	PWM_count , W
	bz		PWM1_full_pulse
	movlw	0x7
	subwf	PWM_count , W
	bz		PWM1_pulse

	movlw	0x4
	subwf	PWM_count , W
	bz		PWM2_full_pulse
	movlw	0x3
	subwf	PWM_count , W
	bz		PWM2_pulse

	rcall	PWM1_clear
	rcall	PWM2_clear

	clrf	CCPR1L				; otherwise, don't generate any pulse
	bcf		CCP1CON,CCP1X
	bcf		CCP1CON,CCP1Y
	movlw	0x12				; relative placement of compass reading just after pulse
	subwf	PWM_count , W		; see if it is time to read the compass
	bz		dsptch_bearing
	return

dsptch_bearing
	decfsz	CMP_count , F		; skip pulses
	return
	movff	CMP_skip, CMP_count
	bsf		CMPS_req
	return

serv_CMPS
	bcf		CMPS_req			; clear the request



read_and_steer
	call	get_bearing			; read the compass
	goto	Steer				; steer

PWM1_pulse							; generate the computed pulse
	movff	PWM1_dc , CCPR1L			; load the current value of pulse width
	bra		PWM2_clear

PWM1_full_pulse				; generate a full pulse
	movlw	0xFF
	movwf	CCPR1L
	bsf		CCP1CON,CCP1X	; don't forget the low order bits
	bsf		CCP1CON,CCP1Y
	bra		PWM2_clear

PWM2_pulse								; generate the computed pulse
	movff	PWM2_dc , CCPR2L			; load the current value of pulse width
	bra		PWM1_clear

PWM2_full_pulse				; generate a full pulse
	movlw	0xFF
	movwf	CCPR2L
	bsf		CCP2CON,CCP2X	; don't forget the low order bits
	bsf		CCP2CON,CCP2Y
	bra		PWM1_clear

PWM1_clear
	clrf	CCPR1L				; otherwise, don't generate any pulse
	bcf		CCP1CON,CCP1X
	bcf		CCP1CON,CCP1Y
	return

PWM2_clear
	clrf	CCPR2L				; otherwise, don't generate any pulse
	bcf		CCP2CON,CCP2X
	bcf		CCP2CON,CCP2Y
	return
	
rudd_cntrl

	bcf		RUD_req
	return

elev_cntrl

	bcf		ELE_req
	return


	GLOBAL serv_PWM, serv_CMPS, PWMInit, elev_cntrl, rudd_cntrl

	end
