

#include "processor.inc"
#include "autop_defines.inc"

;************************************************************************************
;																					*
;	The module for servicing the PWM timer 2 interrupts to generate servo pulses.	*
;																					*
;	The servos require one pulse approximately every 20 milliseconds.				*
;	They cannot be sent much more often than that to the common "analog" servos,	*
;	because the "pulse stretchers" would wind up causing much trouble, including	*
;	short circuiting the power supply. The newer "digital" servos would have been	*
;	easier, but this works well enough.												*
;																					*
;	The position of the servo is determined by the pulse width.						*
;	A 1 millisecond pulse is "hard left", a 2 millisecond pulse is "hard right",	*
;	and "straight ahead" is about 1.5 milliseconds.									*
;																					*
;	So, our job is to create 1 to 2 millisecond pulses every 20 milliseconds.		*
;	The PWM timer 2 is set up to generate an interrupt about every millisecond.		*
;	Most of the time, the interrupt handler simply counts and skips pulses.			*
;	When it is time to send a pulse to the servo, the software creates two pulses	*
;	back to back. The first pulse is a full 1 millisecond pulse. The second pulse	*
;	is a partial pulse, from 0 to 1 milliseconds. The servo sees one pulse, from	*
;	1 to 2 milliseconds long. The pulse width is controlled by the "duty cycle",	*
;	(PWM_dc) such that the pulse width in milliseconds is equal to 1 plus 			*
;	PWM_dc/256.																		*
;																					*
;	This module is also a convenient place to query the compass.					*
;	The best the compass can do is one conversion every 33 milliseconds.			*
;	For our purposes, once every 100 milliseconds is fast enough, so the compass	*
;	is read once every 5th servo pulse.												*
;																					*
;************************************************************************************



	EXTERN	MULS2X1 , MULU2X1 , MULS2X2 , filter
;
;

PROG1 	CODE

;	Initialization code:

PWMInit

	movlw	0xFE		; maximum pulse width minus 1 to make sure there is no gap between pulses.
	movwf	PR2			;initialize PWM period 
	movlw	0x80		;initialize PWM duty cycle for "straight ahead"
	movwf	PWM1_dc
	movwf	CCPR1L
	movwf	PWM2_dc
	movwf	CCPR2L
	clrf	CCP1CON
	clrf	CCP2CON
;
;	Timer 2 is driven off the crystal clock, which is running at 4X4Mhz=16Mhz.
;	Timer 2 sees 1/4 of the crystal clock, or 4Mhz.
;	The prescaler of 16 divides this down to 250Khz, for a small time tic of 4 microseconds.
;	It takes 255 small tics to generate an interrupt, or about once every millisecond:

	movlw	0x03			;postscale 1:1, prescaler 16, Timer2 OFF
	movwf	T2CON

;	in checking the period of radio PWM, I made a mistake and took decimal instead of hex

	movlw	.22			; 22 milliseconds is about right
	movwf	PWM_count
	movwf	PWM_skip

	movlw	0x02
	movwf	xaccelU
	movwf	pitchU
	movwf	yawU
	movwf	refU

	movwf	yawunfH

	movwf	xaccel0U
	movwf	pitch0U
	movwf	yaw0U
	movwf	ref0U

#if ( accelMAV ==1 )
	movlw	0x10
#else
	movlw	0x02
#endif
	movwf	yaccelU
	movwf	yaccel0U

	clrf	xaccelH
	clrf	xaccelL
	clrf	xaccel0H

	clrf	yaccelH
	clrf	yaccelL
	clrf	yaccel0H

	clrf	yaccelmavL
	clrf	yaccelmavH

	clrf	pitchH
	clrf	pitchL
	clrf	pitch0H

	clrf	yawH
	clrf	yawL
	clrf	yaw0H

	clrf	refH
	clrf	refL
	clrf	ref0H

	clrf	yawunfL

	clrf	yawsqU
	clrf	yawsqH
	clrf	yawsqL

	clrf	yawfilU
	clrf	yawfilH
	clrf	yawfilL

	clrf	ruddfilU
	clrf	ruddfilH
	clrf	ruddfilL

	clrf	yawadjH
	clrf	yawadjL

	clrf	ruddunfH
	clrf	ruddunfL

	movlw	0x80
	movwf	pwrudfilH
	clrf	pwrudfilL

	clrf	requests

	movlw	mixgain0
	movwf	mixgain

	movlw	mixgyr0
	movwf	mixgyr

	movlw	pitchgain0
	movwf	pitchgain

	movlw	accelgain0
	movwf	accelgain

	movlw	yawgain0
	movwf	yawgain

	movlw	auggain0
	movwf	auggain

	return

PWM_clear
	rcall	PWM1_clear
	rcall	PWM2_clear
	return

sel_vref
	movlw	vrefAD
	movwf	ADCON0
	return

read_vref
	bsf		ADCON0,GO

	movf	refH , w
	subwf	refL , f

	movf	refU , w
	subwfb	refH , f

	clrf	WREG
	subwfb	refU , f


cnvrt_ref
	btfsc	ADCON0,GO
	bra		cnvrt_ref

	movf	ADRESL , w
	addwf	refL , f

	movf	ADRESH , w
	addwfc	refH , f

	clrf	WREG
	addwfc	refU , f

	btfss	calibref
	return
	bcf		calibref
	movff	refU , ref0U
	movff	refH , ref0H
	return

sel_xaccel
	movlw	xaccelAD	;	xaccel
	movwf	ADCON0
	return


read_xaccel
	bsf		ADCON0,GO

#if ( filteraccel == 1)

	movf	xaccelH , w
	subwf	xaccelL , f

	movf	xaccelU , w
	subwfb	xaccelH , f

	clrf	WREG
	subwfb	xaccelU , f
#else
	nop
	nop
	nop
	nop
#endif

cnvrt_xaccel
	btfsc	ADCON0,GO
	bra		cnvrt_xaccel

#if ( filteraccel == 1)

	movf	ADRESL , w
	addwf	xaccelL , f

	movf	ADRESH , w
	addwfc	xaccelH , f

	clrf	WREG
	addwfc	xaccelU , f

#else
	movf	ADRESL , w
	movwf	xaccelH
	
	movf	ADRESH , w
	movwf	xaccelU

	clrf	WREG
	movwf	xaccelL

#endif


	btfss	calibxaccel
	return
	bcf		calibxaccel
	movff	xaccelU , xaccel0U
	movff	xaccelH , xaccel0H
	return

sel_yaccel
	movlw	yaccelAD	;	yaccel
	movwf	ADCON0
	return

read_yaccel
	bsf		ADCON0,GO

#if ( filteraccel == 1 )

	movf	yaccelH , w
	subwf	yaccelL , f

	movf	yaccelU , w
	subwfb	yaccelH , f

	clrf	WREG
	subwfb	yaccelU , f
#else
	nop
	nop
	nop
	nop
#endif

cnvrt_yaccel
	btfsc	ADCON0,GO
	bra		cnvrt_yaccel

	movf	ADRESL , w
	addwf	yaccelmavL , f

	movf	ADRESH , w
	addwfc	yaccelmavH , f

#if ( filteraccel == 1 )

	movf	yaccelmavL , w
	addwf	yaccelL , f

	movf	yaccelmavH , w
	addwfc	yaccelH , f

	clrf	WREG
	addwfc	yaccelU , f

#else

	movf	yaccelmavL , w
	movwf	yaccelH

	movf	yaccelmavH , w
	movwf	yaccelU

	clrf	WREG
	movwf	yaccelL

#endif

	clrf	yaccelmavL
	clrf	yaccelmavH

	btfss	calibyaccel
	return
	bcf		calibyaccel
	movff	yaccelU , yaccel0U
	movff	yaccelH , yaccel0H
	return

mav_yaccel

#if ( accelMAV == 1)

	bsf		ADCON0,GO
	nop
	nop
	nop
	nop

cnvrt_yaccel2
	btfsc	ADCON0,GO
	bra		cnvrt_yaccel2

	movf	ADRESL , w
	addwf	yaccelmavL , f

	movf	ADRESH , w
	addwfc	yaccelmavH , f

	return
#else
	return
#endif

sel_pitch
	movlw	pitchAD	;	pitch
	movwf	ADCON0
	return


read_pitch
	bsf		ADCON0,GO

	movf	pitchH , w
	subwf	pitchL , f

	movf	pitchU , w
	subwfb	pitchH , f

	clrf	WREG
	subwfb	pitchU , f

cnvrt_pitch
	btfsc	ADCON0,GO
	bra		cnvrt_pitch

	movf	ADRESL , w
	addwf	pitchL , f

	movf	ADRESH , w
	addwfc	pitchH , f

	clrf	WREG
	addwfc	pitchU , f

	btfss	calibpitch
	return
	bcf		calibpitch
	movff	pitchU , pitch0U
	movff	pitchH , pitch0H
	return

sel_yaw
	movlw	yawAD	;	yaw
	movwf	ADCON0
	return

read_yaw
	bsf		ADCON0,GO

	movf	yawH , w
	subwf	yawL , f

	movf	yawU , w
	subwfb	yawH , f

	clrf	WREG
	subwfb	yawU , f

cnvrt_yaw
	btfsc	ADCON0,GO
	bra		cnvrt_yaw

	movf	ADRESL , w
	movwf	yawunfL
	addwf	yawL , f

	movf	ADRESH , w
	movwf	yawunfH
	addwfc	yawH , f

	clrf	WREG
	addwfc	yawU , f

	btfss	calibyaw
	return
	bcf		calibyaw
	movff	yawU , yaw0U
	movff	yawH , yaw0H
	return

read_yaw1
	rcall	read_yaw
	return

read_yaw2

	rcall	read_yaw
	return

;
;	service the PWM interrupt once every millisecond
serv_PWM
	bcf		PIR1, TMR2IF		; turn off the interrupt flag
	dcfsnz	PWM_count , F		; decrement the pulse count
	movff	PWM_skip , PWM_count	; reload the skip counter

	movlw	low	PWM_goto_table
	movwf	PWM_goto_L

	movf	PWM_count, W
	decf	WREG , W
	addwf	WREG , W

	addwf	PWM_goto_L , F

	movlw	high PWM_goto_table
	movwf	PCLATH 
	clrf	WREG
	addwfc	PCLATH , F

	movf	PWM_goto_L , W
	movwf	PCL


PWM_goto_table
	
	bra		task_00	
	bra		task_01	
	bra		task_02	
	bra		task_03	
	bra		task_04	
	bra		task_05	
	bra		task_06	
	bra		task_07	
	bra		task_08
	bra		task_09	
	bra		task_10		
	bra		task_11
	bra		task_12		
	bra		task_13		
	bra		task_14		
	bra		task_15		
	bra		task_16		
	bra		task_17		
	bra		task_18		
	bra		task_19		
	bra		task_20		
	bra		task_21	
	bra		task_22	
	

task_22
	
	return

task_21
		
	bra		sel_vref	

task_20
		
	call	read_vref
	bra		sel_yaccel

task_19
	
	call	read_yaccel
	bra		sel_xaccel

task_18
	
	call	read_xaccel
	bra		sel_yaccel	


task_17
		
	call	mav_yaccel
	bra		sel_pitch

task_16
		
	call	read_pitch
	bra		sel_yaccel

task_15
		
	call	mav_yaccel
	bra		sel_yaw

task_14
		
	call	read_yaw1
	bra		sel_yaccel

task_13

	call	mav_yaccel
	call	sel_yaw
	bra		filter_mix

task_12

	call	read_yaw2
	bra		sel_yaccel

task_11
	call	mav_yaccel	
	bra		compute_elevator

task_10
	
	bra		filter_pwrud

task_09

	call	mav_yaccel
	bra		compute_rudder

task_08

	bra		scale_gains

task_07
	
	call	mav_yaccel
	bra		PWM1_full_pulse	

task_06
	
	bra		PWM1_pulse

task_05
	
	call	mav_yaccel
	bra		PWM_clear

task_04

	return

task_03
	
	bra		PWM2_full_pulse	

task_02
	
	bra		PWM2_pulse

task_01
	
	bra		PWM_clear

task_00
	
	return




PWM1_pulse							; generate the computed pulse
	movff	PWM1_dc , CCPR1L			; load the current value of pulse width
	bra		PWM2_clear

PWM1_full_pulse				; generate a full pulse
	movlw	0xFF
	movwf	CCPR1L
	bsf		CCP1CON,CCP1X	; don't forget the low order bits
	bsf		CCP1CON,CCP1Y
	bra		PWM2_clear

PWM2_pulse							; generate the computed pulse
	movff	PWM2_dc , CCPR2L			; load the current value of pulse width
	bra		PWM1_clear

PWM2_full_pulse				; generate a full pulse
	movlw	0xFF
	movwf	CCPR2L
	bsf		CCP2CON,CCP2X	; don't forget the low order bits
	bsf		CCP2CON,CCP2Y
	bra		PWM1_clear

PWM1_clear
	clrf	CCPR1L				; otherwise, don't generate any pulse
	bcf		CCP1CON,CCP1X
	bcf		CCP1CON,CCP1Y
	return

PWM2_clear
	clrf	CCPR2L				; otherwise, don't generate any pulse
	bcf		CCP2CON,CCP2X
	bcf		CCP2CON,CCP2Y
	return


compute_elevator

	bsf		ELE_req
	return

compute_rudder

	bsf		RUD_req
	return

filter_mix

	movlw	banktau
	movwf	taufil

	movff	yawadjH , infilH
	movff	yawadjL , infilL
	movff	yawfilU , outfilU
	movff	yawfilH , outfilH
	movff	yawfilL , outfilL
	call	filter
	movff	outfilU , yawfilU
	movff	outfilH , yawfilH
	movff	outfilL , yawfilL

	movff	ruddunfH , infilH
	movff	ruddunfL , infilL
	movff	ruddfilU , outfilU
	movff	ruddfilH , outfilH
	movff	ruddfilL , outfilL
	call	filter
	movff	outfilU , ruddfilU
	movff	outfilH , ruddfilH
	movff	outfilL , ruddfilL

	return

filter_pwrud

	btfss	radio_on		;	filter if radio is on
	return

	movf	pwrudfilH , w
	subwf	pwrudfilL , f
	
	clrf	WREG
	subwfb	pwrudfilH , f

	movf	pwrudin , w
	addwf	pwrudfilL , f

	clrf	WREG
	addwfc	pwrudfilH , f

	return	

scale_gains

	btfss	radio_on
	bra		no_scaling

	scale 	pwselin , mixgain0 , mixgain
	scale	pwselin , mixgyr0 , mixgyr
	scale	pwselin , pitchgain0 , pitchgain
	scale	pwselin , accelgain0 , accelgain
	scale	pwselin , yawgain0 , yawgain
	scale	pwselin , steering_gain0 , strngGain
	scale	pwselin , auggain0 , auggain
	
	bra	serv_rev

no_scaling

	movlf	mixgain0 , mixgain
	movlf	mixgyr0 , mixgyr
	movlf	pitchgain0 , pitchgain
	movlf	accelgain0 , accelgain
	movlf	yawgain0 , yawgain
	movlf	steering_gain0 , strngGain
	movlf	auggain0 , auggain

serv_rev

#ifdef	SWR1
	btfss	SWR1
	bra		serv2_rev
	negf	yawgain
	negf	strngGain

#endif

serv2_rev

#ifdef	SWR2
	btfss	SWR2
	return
	negf	pitchgain
	negf	accelgain
	negf	mixgain
	negf	mixgyr

#endif

	return


	GLOBAL serv_PWM, PWMInit

	end
