

#include "processor.inc"
#include "autop_defines.inc"

;************************************************************************************
;																					*
;		Routines for servicing the GPS through the Universal Serial	Asynchronous	*
;		Receiver Transmitter (USART).												*
;																					*
;		Routines include:															*
;																					*
;		GPSInit: 	initialize the USART and GPS variables.							*
;		serv_GPS:	service the USART receiver interrupts.							*
;		set_gxx:	send commands to the GPS to turn off uneeded messages.			*
;		GPS_out:	used by set_gxx to send one command to GPS.					*
;		display_GPS:display the latest GPS info, plus the compass direction.		*
;		parse_GPS:	parse the GPS message into its pieces.							*
;																					*
;		plus, there are other routines used internally by the above routines.		*
;																					*
;		The reception of a character from the GPS by the USART triggers an interrupt*
;		The service routine looks at each character to decide what to do.			*
;		The "$" character at the beginning of a message is used as a start synch.	*
;		The "\n" character at the end of a message is used as an end synch.			*
;		In between, characters are placed in a buffer.								*
;		One complete message is expected once per second, and is used to kick off	*
;		the one-second processing.													*
;																					*
;************************************************************************************



	EXTERN	LCDInit, d_write, i_write, LCDLine_1, LCDLine_2, bin_bcd, bin16_bcd
	EXTERN	stan_char_1, stan_char_2, delay_1s
	EXTERN	one_sec_tasks, end_dsp_GPS , end_dsp_XY , end_dsp_vtg
	EXTERN	rect_to_polar

PROG1 	CODE


;	The following are specific command strings that could be used to turn off
;	ALL of the messages that the GPS sends. Most of the GPS messages are not needed.
;	set_gxx turns off all but one message type, by sending out a subset of the strings.
;	Currently, we turn off gll, gsa, gsv, and rmc, leaving gga.
;	gga contains longitude, latitude, and altitude information.
;	gga does not contain speed and direction, which is is rmc.

;set_gga
;	data	"$PSRF103,00,00,00,01*24\r\n" ; gga off

set_gga
	data	"$PSRF103,00,00,01,01*25\r\n" ; gga on
	
set_gll
	data	"$PSRF103,01,00,00,01*25\r\n"
set_gsa
	data	"$PSRF103,02,00,00,01*26\r\n"
set_gsv
	data	"$PSRF103,03,00,00,01*27\r\n"
set_rmc
	data	"$PSRF103,04,00,00,01*20\r\n"

;set_vtg
;	data	"$PSRF103,05,00,00,01*21\r\n" ; vtg off

set_vtg
	data	"$PSRF103,05,00,01,01*20\r\n" ; vtg on


;
;	Initialize the GPS control:

GPSInit

	LFSR	FSR0 ,	GPS_buffer	;	point to the GPS receiver buffer
	movff	FSR0L , GPS_bf_pnt	;	save the low portion of the pointer between interrupts
	clrf	state_flags			;	clear the state control bits
	clrf	xFocusH
	clrf	xFocusL
	clrf	yFocusH
	clrf	yFocusL

#if	( GPSDEBUG == 1)
	movlw	B'00000000'	;
#else
	movlw	B'10100100'		;initialize USART transmit
#endif

	movwf	TXSTA			;8-bit, Async, High Speed
							;1=CSRC=x
							;0=TX9=8bit
							;1=TXEN=enabled
							;0=SYNC=asynch mode
							;0=unused
							;1=BRGH=high speed baud
							;0=read only
							;0=TX9D
	movlw	ubrg_4p8_16		; 4.8 kbaud at 16MHz
	movwf	SPBRG	
		;

#if	( GPSDEBUG == 1)
	movlw	B'00000000'	;
#else
	movlw	B'10000000'	;initialize USART receive
#endif

	movwf	RCSTA			;8-bit
							;1=SPEN= TX and RX pins enabled
							;0=RX9=8bit
							;0=unused
							;0=CREN=receiver enabled (don't turn it on until ready)
							;0=unused
							;0=FERR=read only framing error
							;0=OERR=read only overrun error
							;0=RX9D
	return

;	The routine that gets called to service a GSP generated USART interrupt:

serv_GPS
	bcf		PIR1, RCIF			;	reset the interrupt flag
	btfsc	RCSTA , FERR		;	check for a framing error
	rcall	GPS_ferr
	btfsc	RCSTA , OERR		;	check for an overrun error
	bra		GPS_oerr
								;	otherwise, normal reception
	movff	RCREG, GPS_char		;	fetch the character from the receiver

	movff	GPS_bf_pnt , FSR0L	;	restore the buffer pointer from last time

	movlw	'$'					;	check for "$" at the start of the message
	subwf	GPS_char, W
	bnz		GPS_n_dollar
	LFSR	FSR0 , GPS_buffer	;	reset the buffer pointer at the start of the message
	bsf		GPS_dollar			;	set the GPS_dollar control bit to indicate start

GPS_n_dollar

	movlw	0x0A				;	check for \n at the end of a message
	subwf	GPS_char, W
	bnz		GPS_n_end

	movff	FSR0L , GPS_bf_pnt	;	save the buffer pointer for next time

	bsf		GPS_req				;	request GPS completion processing
	return


GPS_n_end
	
	btfsc	GPS_dollar			;	don't save anything until the start "$" is recognized.
								;	This must be done since the CPU can start up in the middle
								;	of a GSP message.
	movff	GPS_char, POSTINC0	;	Finally! Save the character to the buffer and increment.
	movff	FSR0L , GPS_bf_pnt	;	save the buffer pointer for next time
	return


;	Framing and receiver errors must be cleared to avoid "hanging" the CPU.
;	In particular, an uncleared overrun error will regenerate an interrupt
;	as soon as interrupts are re-enabled, effectively locking up the CPU.
;	Although carefull design would seem to preclude these errors, you never know.

GPS_ferr						;	Framing error: clear by reading the receiver register.
	movff	RCREG, GPS_char
	btfss	RCSTA , OERR
	return
GPS_oerr						;	Overrun error: clear by toggling the receiver enable.
	bcf		RCSTA , CREN
	nop
	bsf		RCSTA , CREN
	return


;************* silence the unneeded GPS messages

set_gxx

	bcf			GPS_verbose
	ld_tblptr	set_rmc	;	point to the message
	rcall		GPS_out		;	send it out
	ld_tblptr	set_gll
	rcall		GPS_out
	ld_tblptr	set_gsa
	rcall		GPS_out
	ld_tblptr	set_gsv
	rcall		GPS_out
	ld_tblptr	set_vtg
	rcall		GPS_out

	return


;************** GPS_out ******************
;              sends a \n terminated string out to GPS

GPS_out

	tblrd	*+				;	read the table with post increment
	movf	TABLAT,W		;	move from the table latch to the W register
	movwf	TXREG			;	move into the transmitter register
	btfss	TXSTA,TRMT		;	wait for data to go out
	bra		$-2
	sublw	'\n'			;	look for the end of the string
	bnz		GPS_out
	return

;************** Display GPS info on the LCD *************

display_GPS					;	entry point from the menu driver
	bcf		VTG_end			;	wait for next end to synch
#ifdef	scroll
	btfss	scroll			;	wait for RA4 release
	bra		$-2	
#endif
#ifdef	select	
	btfss	select			;	wait for RB0 release
	bra		$-2
#endif

disp_GPS_again				;	display loop

#ifdef	select
	btfss	select
	rcall	enable_org
#endif

	btfsc	VTG_end			;	look for the end of the message coming in!

	rcall	LCD_dsp_GPS		;	display the long, lat, direction and satellite data
#ifdef	scroll
	btfsc	scroll			;	presss RA4 to get out
	bra		disp_GPS_again
	btfss	scroll			;	release RA4 to complete
	bra		$-2
#endif
	goto	end_dsp_GPS

;************** Display vtg info on the LCD *************

display_vtg					;	entry point from the menu driver
	bcf		VTG_end			;	wait for next end to synch
#ifdef	scroll
	btfss	scroll			;	wait for RA4 release
	bra		$-2	
#endif
#ifdef	select	
	btfss	select			;	wait for RB0 release
	bra		$-2
#endif

disp_vtg_again				;	display loop


	btfsc	VTG_end			;	look for the end of the message coming in!

	rcall	LCD_dsp_vtg		;	display the GPS heading and speed
#ifdef	scroll
	btfsc	scroll			;	presss RA4 to get out
	bra		disp_vtg_again
	btfss	scroll			;	release RA4 to complete
	bra		$-2
#endif
	goto	end_dsp_vtg


;************** Display xy info on the LCD *************

display_XY					;	entry point from the menu driver
	bcf		VTG_end			;	wait for next end to synch
#ifdef	scroll
	btfss	scroll			;	wait for RA4 release
	bra		$-2	
#endif
#ifdef	select	
	btfss	select			;	wait for RB0 release
	bra		$-2
#endif

disp_XY_again				;	display loop
#ifdef	select
	btfss	select
	bsf		enable_focus
#endif

	btfsc	VTG_end			;	look for the end of the message coming in!
	rcall	LCD_dsp_XY		;	display the long, lat, direction and satellite data
#ifdef	scroll
	btfsc	scroll			;	presss RA4 to get out
	bra		disp_XY_again
	btfss	scroll			;	release RA4 to complete
	bra		$-2
#endif
	goto	end_dsp_XY

enable_org
#ifdef	select
	btfss	select
	bra		$-2
	bsf 	GPS_rq_org
#endif
	return

complete_GPS
	bcf		GPS_req				;	clear the request
	rcall	parse_GPS			;	parse the GPS message into its pieces
	btfss	GPS_valid
	return

	bcf		GPS_valid
	movf	GPS_fix , W
	sublw	'0'
	bz		GPS_not_tracking
	bsf		GPS_tracking
call_one_sec
	btfss	nav_busy
	call	one_sec_tasks
	return

GPS_not_tracking
	btfss	test_mode
	return
	bra		call_one_sec

parse_GPS					;	break the GPS gga message into its pieces.
							;   Call skip_GPS_item to skip an item in the buffer.
							;	Call read_GPS_item to parse an item from the buffer.


	movff	GPS_buffer+4 , WREG	;	check if this is a GGA message
	sublw	'G'
	bz		parse_gga

	movff	GPS_buffer+4 , WREG	;	check if this is a GGA message
	sublw	'T'
	bz		parse_vtg

	bra	GPS_is_verbose


parse_vtg

	LFSR	FSR0 , GPS_buffer	;	set the FSR0 pointer to the GPS buffer
	movlw	7+1
	movwf	GPS_item_size
	rcall	skip_GPS_item		; 	skip message ID

	LFSR	FSR1 , GPS_course	;	read true heading
	movlw	GPS_courseR+1
	movwf	GPS_item_size
	rcall	read_GPS_item

	movlw	3					; 	skip "T"
	movwf	GPS_item_size
	rcall	skip_GPS_item

	movlw	GPS_courseR+1		;	skip magnetic heading
	movwf	GPS_item_size
	rcall	skip_GPS_item

	movlw	3					; 	skip "M"
	movwf	GPS_item_size
	rcall	skip_GPS_item

	movlw	GPS_speedR+1		;	skip speed in nauts
	movwf	GPS_item_size
	rcall	skip_GPS_item

	movlw	3					; 	skip "N"
	movwf	GPS_item_size
	rcall	skip_GPS_item

	LFSR	FSR1 , GPS_speed
	movlw	GPS_speedR+1
	movwf	GPS_item_size
	rcall	read_GPS_item

	bsf		GPS_valid

	bra		convert_vtg


parse_gga

	LFSR	FSR0 , GPS_buffer	;	set the FSR0 pointer to the GPS buffer
	movlw	7+1
	movwf	GPS_item_size
	rcall	skip_GPS_item		; 	skip message ID
	movlw	.11+1
	movwf	GPS_item_size
	rcall	skip_GPS_item		; 	skip UTC time
	LFSR	FSR1 , GPS_lat  	; 	point the FSR1 pointer to latitude
	movlw	GPS_latR+1
	movwf	GPS_item_size
	rcall	read_GPS_item		;	move the data
	LFSR	FSR1 , GPS_NS		;	read NS
	movlw	GPS_NSR+1
	movwf	GPS_item_size
	rcall	read_GPS_item		; 
	LFSR	FSR1 , GPS_long 	; 	read longitude
	movlw	GPS_longR+1
	movwf	GPS_item_size
	rcall	read_GPS_item
	LFSR	FSR1 , GPS_EW		;	read EW
	movlw	GPS_EWR+1
	movwf	GPS_item_size
	rcall	read_GPS_item
	LFSR	FSR1 , GPS_fix		;	position fix code
	movlw	GPS_fixR+1
	movwf	GPS_item_size
	rcall	read_GPS_item
	LFSR	FSR1 , GPS_sat		; 	read number of satellites
	movlw	GPS_satR+1
	movwf	GPS_item_size
	rcall	read_GPS_item
	movlw	6+1
	movwf	GPS_item_size
	rcall	skip_GPS_item		; 	skip HDOP
	LFSR	FSR1 , GPS_alt		; 	read altitude
	movlw	GPS_altR+1
	movwf	GPS_item_size
	rcall	read_GPS_item

;	bsf		GPS_valid			;	valid message

	rcall	asc2bin				;	convert from ascii to binary

	btfss	GPS_rq_org			;	check for origin capture request
	return

	bcf		GPS_rq_org			;	clear the request
	bcf		GPS_org_rdy			;	origin not valid
	
	LFSR	FSR0 , GPS_long		;	record the origin longitude
	LFSR	FSR1 , orig_long
	movlw	orig_longR+1
	movwf	GPS_item_size
	rcall	read_GPS_item

	LFSR	FSR0 , GPS_lat		; 	record the origin latitude
	LFSR	FSR1 , orig_lat
	movlw	orig_latR+1
	movwf	GPS_item_size
	rcall	read_GPS_item

	movf	actualDir , W
	movwf	origDir	;	record the origin direction
	movwf	desiredDir

	movff	GPS_lat_bin , orig_lat_bin
	movff	GPS_lat_bin+1 , orig_lat_bin+1
	movff	GPS_long_bin , orig_long_bin
	movff	GPS_long_bin+1 , orig_long_bin+1

	bsf		GPS_org_rdy				;	signal new origin is ready
	
	return

;
;	response if GPS is yakking:

GPS_is_verbose
;	bsf	GPS_verbose					;	set the flag to turn off the GPS extra messages
	pop
	return

;	Skip an item in the GPS buffer by looking for the next comma:

skip_GPS_item
	dcfsnz	GPS_item_size
	bra		GPS_is_verbose
	movf	POSTINC0 , W
	sublw	','
	bnz		skip_GPS_item
	return

;	Move an item from the GSP buffer to the slot pointed to by FSR1:

read_GPS_item
	dcfsnz	GPS_item_size
	bra		GPS_is_verbose
	movf	POSTINC0 , W	;	get a character from the buffer and increment pointer
	movwf	POSTINC1		;	put it into the slot and increment pointer
	sublw	','				;	look for terminating comma
	bnz		read_GPS_item
	return

;	Display some GPS and compass information:

LCD_dsp_GPS
	bcf		VTG_end					;	clear the handshaking bit for the next message
	call	LCDLine_1				;	point LCD cursor to beginning of line 1
	LFSR	FSR1 , GPS_lat			;	point to the latitude slot
	movlw	GPS_latR+1
	movwf	GPS_item_size
	rcall	disp_item				;	display on the LCD


;	LFSR	FSR1 , GPS_alt
;	movlw	GPS_altR
;	movwf	GPS_item_size+1
;	call	disp_item

	movf	actualDir , W			;	get the compass reading
	call	bin_bcd					;	convert to ASCII

	movf	MSD , W					;	display first character of direction
	movwf	temp_wr
	call	d_write

	movf	MsD , W					;	display second character of direction
	movwf	temp_wr
	call	d_write

	movf	LSD , W					;	display third charater of direction
	movwf	temp_wr
	call	d_write

	call	LCDLine_2				;	set LCD cursor to start of second line
	LFSR	FSR1 , GPS_long			;	point to longitude
	movlw	GPS_longR+1
	movwf	GPS_item_size
	rcall	disp_item				;	display longitude
	LFSR	FSR1 , GPS_sat			;	point to latitude
	movlw	GPS_satR+1
	movwf	GPS_item_size
	rcall	disp_item				;	display latitude
	LFSR	FSR1 , GPS_fix			;   point to GPS fix data
	movlw	GPS_fixR+1
	movwf	GPS_item_size
	rcall	disp_item				;   display fix data
	return

bcdDigOut	macro	digit
	movf	digit , W
	rcall	bcd2ascOut
	endm


flipSign
	movlw	'-'
	movwf	Sign
	negf	NumL
	comf	NumH , F
	movlw	.0
	addwfc	NumH , F
	return



bcd2ascOut
	addlw	'0'				
	movwf	temp_wr
	goto	d_write

bin16out

	movlw	'+'
	movwf	Sign
	btfsc	NumH , 7
	rcall	flipSign

	call	bin16_bcd
	
	movf	Sign , W
	movwf	temp_wr
	call	d_write

	bcdDigOut	TenK
	bcdDigOut	Thou
	bcdDigOut	Hund
	bcdDigOut	Tens
	bcdDigOut	Ones

	movlw	' '
	movwf	temp_wr
	call	d_write

	return

LCD_dsp_vtg
	bcf		VTG_end					;	clear the handshaking bit for the nesxt message


	movlw	.0			; display startup announcement line 1
	movwf	ptr_pos
	call	stan_char_1

	movlw	.0			; display startup announcement line 1
	movwf	ptr_pos
	call	stan_char_2


	call	LCDLine_1				;	point LCD cursor to beginning of line 1
	LFSR	FSR1 , GPS_course			;	point to the course
	movlw	GPS_courseR+1
	movwf	GPS_item_size
	rcall	disp_item				;	display on the LCD


	call	LCDLine_2				;	set LCD cursor to start of second line
	LFSR	FSR1 , GPS_speed			;	point to longitude
	movlw	GPS_speedR+1
	movwf	GPS_item_size
	rcall	disp_item				;	display longitude

	return

save_focus
	bcf		enable_focus

	movf	xWorldH , W
	movwf	xFocusH
	movwf	X_rectH

	movf	xWorldL , W
	movwf	xFocusL
	movwf	X_rectL

	movf	yWorldH , W
	movwf	yFocusH
	movwf	Y_rectH

	movf	yWorldL , W
	movwf	yFocusL
	movwf	Y_rectL

;	used only in the truck:

#if ( truck == 1)
	call rect_to_polar

	movff X_rectH , ellipseConstH
	movff X_rectL , ellipseConstL

	movlw	minRadiusX4

	addwf ellipseConstL , f
	movlw .00
	addwfc ellipseConstH , f

#endif

	return

;	Display some GPS and compass information:

LCD_dsp_XY
	bcf		VTG_end					;	clear the handshaking bit for the next message

	btfsc	enable_focus
	rcall	save_focus

	movlw	.0			; display startup announcement line 1
	movwf	ptr_pos
	call	stan_char_1

	movlw	.0			; display startup announcement line 1
	movwf	ptr_pos
	call	stan_char_2

	call	LCDLine_1				;	point LCD cursor to beginning of line 1

	movff	xWorldH , NumH
	movff	xWorldL , NumL

	rcall	bin16out

	call	LCDLine_2				

	movff	yWorldH , NumH
	movff	yWorldL , NumL

	rcall	bin16out
	
	return

;	Issue a space. Sometimes used to clear the end of variable length line.

LCDspace
	movlw	0x20
	movwf	temp_wr
	call	d_write
	return

;	Display a comma terminated string on the LCD.
;	FSR1 points to the string.

disp_item
	dcfsnz	GPS_item_size
	bra		GPS_is_verbose
	movff	POSTINC1 , GPS_ps_char	;	get a character, increment pointer
	movf	GPS_ps_char , W
	movwf	temp_wr					;	display expects character in temp_wr
	call	d_write					;	display the character on LCD
	movlw	','						;	check for comma
	subwf	GPS_ps_char , W
	bnz		disp_item				;	repeat until the comma
	return


asc2bin								;	multiply 5 l.s.digits by scale factor and convert to 16bit binary
									;	scale factors are 37 for latitude, 27 for longitude, to convert
									;	to units 1/2 meters at our latitude, 42 degrees

	clrf	GPS_lat_bin				;	clear the MSbyte
	movf	GPS_lat+lat_d1, W		;	load d1
	addlw	0xD0					;	convert from ascii to binary
	mullw	.10						;	multiply by 10
	movf	GPS_lat+lat_d0, W		;	load d0
	addlw	0xD0					;	convert to binary
	addwf	PRODL, W				;	add 10*d1
	mullw	((LatGain*.256+.50)/.100)	;	multiply by latitude gain
	movf	PRODH, W				;	divide by 256
	movwf	GPS_lat_bin+1			;	load into LSbyte of result
	movf	GPS_lat+lat_d3 , W		;	load d3
	addlw	0xD0					;	convert to binary
	mullw	.10						;	multiply by 10
	movf	GPS_lat+lat_d2, W		;	load d2
	addlw	0xD0					;	convert to binary
	addwf	PRODL, W				;	add 10*d3
	mullw	LatGain					;	multiply by latitude gain
	movf	PRODL, W
	addwf	GPS_lat_bin+1, F		;	add to LSbyte of result
	movf	PRODH, W
	addwfc	GPS_lat_bin , F			;	add to MSbyte of result
	movf	GPS_lat+lat_d4, W		;	load d4
	addlw	0xD0					;	convert to binary
	mullw	.25						;	multiply by 25 (100/4)
	movf	PRODL, W
	mullw	.4*LatGain					;	multiply by 4 time latitude gain
	movf	PRODL, W
	addwf	GPS_lat_bin+1, F		;	add to reslt
	movf	PRODH, W
	addwfc	GPS_lat_bin, F

									;	similar for longitude, except different constants

	clrf	GPS_long_bin
	movf	GPS_long+long_d1, W
	addlw	0xD0
	mullw	.10
	movf	GPS_long+long_d0, W
	addlw	0xD0
	addwf	PRODL, W
	mullw	((LongGain*.256+.50)/.100)
	movf	PRODH, W
	movwf	GPS_long_bin+1
	movf	GPS_long+long_d3 , W
	addlw	0xD0
	mullw	.10
	movf	GPS_long+long_d2, W
	addlw	0xD0
	addwf	PRODL, W
	mullw	LongGain
	movf	PRODL, W
	addwf	GPS_long_bin+1, F
	movf	PRODH, W
	addwfc	GPS_long_bin , F
	movf	GPS_long+long_d4, W
	addlw	0xD0
	mullw	.25
	movf	PRODL, W
	mullw	.4*LongGain
	movf	PRODL, W
	addwf	GPS_long_bin+1, F
	movf	PRODH, W
	addwfc	GPS_long_bin, F

	negf	GPS_long_bin+1
	comf	GPS_long_bin , F
	movlw	0x00
	addwfc	GPS_long_bin


	return

vtg_null

	bsf		VTG_end
	return

max_speed
;
;	max speed = 999 decimal, 03E7H
;
	movlw	0x03
	movwf	speedH
	movlw	0xE7
	movwf	speedL
	bsf		VTG_end
	return
;
;
;
convert_vtg
;

	LFSR	FSR0 , GPS_course 
	LFSR	FSR1 , GPS_course+(GPS_courseR-2)
	rcall	measure_item
	movf	VTG_item_size , W
	dcfsnz	WREG
	bra		vtg_null
	sublw	GPS_courseR-2
	bz		skip_course_just

	movf	VTG_item_size , W
	rcall	move_item
	decf	VTG_item_size
	negf	VTG_item_size
	movlw	GPS_courseR-2
	addwf	VTG_item_size , F
	lfsr	FSR0 , GPS_course
	rcall	left_pad

skip_course_just

	movlw	'0'

#ifdef  version1 
#define crsedig0 4
#define crsedig1 2
#define crsedig2 1
#define crsedig3 0
#endif

#ifdef  version2 
#define crsedig0 5
#define crsedig1 3
#define crsedig2 2
#define crsedig3 1
#endif

#ifdef	sf252v1
#define crsedig0 4
#define crsedig1 2
#define crsedig2 1
#define crsedig3 0
#endif

#ifdef	sf2520v1
#define crsedig0 4
#define crsedig1 2
#define crsedig2 1
#define crsedig3 0
#endif

#ifdef	sf2520v1a
#define crsedig0 4
#define crsedig1 2
#define crsedig2 1
#define crsedig3 0
#endif

	subwf	GPS_course+crsedig0, F
	subwf	GPS_course+crsedig1 , F
	subwf	GPS_course+crsedig2 , F
	subwf	GPS_course+crsedig3 , F

;	use the speed storage for temporary 16 bit arithmetic

	movlw	.18

	mulwf	GPS_course+crsedig0

	movff	PRODL , speedL
	clrf	speedH
	movlw	.182

	mulwf	GPS_course+crsedig1

	movf	PRODL , W
	addwf	speedL , F
	movf	PRODH , W
	addwfc	speedH , F
	movff	speedH , course
	movlw	.7

	mulwf	GPS_course+crsedig2

	movf	PRODL , W
	addwf	course , F
	movlw	.71

	mulwf	GPS_course+crsedig3

	movf	PRODL , W
	addwf	course , F

	negf	course
	movlw	.64
	addwf	course , F

	movff	course , actualDir
 

	movlw	'0'
	addwf	GPS_course+crsedig0 , F
	addwf	GPS_course+crsedig1 , F
	addwf	GPS_course+crsedig2 , F
	addwf	GPS_course+crsedig3 , F


	LFSR	FSR0 , GPS_speed
	LFSR	FSR1 , GPS_speed+(GPS_speedR-3) ; there is an extra, extra byte for over 99.9, nauts
	rcall	measure_item
	movf	VTG_item_size , W
	dcfsnz	WREG
	bra		vtg_null
	sublw	GPS_speedR-2
	bz		max_speed
	movf	VTG_item_size , W
	sublw	GPS_speedR-2
	bz		skip_speed_just

	movf	VTG_item_size , W	
	rcall	move_item
	decf	VTG_item_size
	negf	VTG_item_size
	movlw	GPS_speedR-3
	addwf	VTG_item_size , F
	lfsr	FSR0 , GPS_speed
	rcall	left_pad

skip_speed_just

	movlw	'0'
	subwf	GPS_speed , F
	subwf	GPS_speed+1 , F
	subwf	GPS_speed+3 , F

	clrf	speedH
	movff	GPS_speed+3 , speedL
	movlw	.10
	mulwf	GPS_speed+1
	movf	PRODL , W
	addwf	speedL , F
	movlw	.100
	mulwf	GPS_speed
	movf	PRODL , W
	addwf	speedL , F
	movf	PRODH , W
	addwfc	speedH , F

	movlw	'0'
	addwf	GPS_speed , F
	addwf	GPS_speed+1 , F
	addwf	GPS_speed+3 , F


	bsf		VTG_end
	return

;
;
;	measure_item : determines the actual length of a GPS item.
;	inputs: FSR0 points to the comma terminated item.
;	output: FSR0 points to the comma, VTG_item_size = actual length of item, including the comma
;

measure_item
	clrf	VTG_item_size
next_item_char
	incf	VTG_item_size
	movf	POSTINC0 , W
	sublw	','
	bz		end_of_item
	bra		next_item_char
end_of_item
	movf	POSTDEC0 , W
	return

;
;	move_item : moves an item as part of right justification
;	inputs: FSR0 points to the comma of a variable length GPS item
;			FSR1 points to the far right of the fixed length item
;			WREG is the number of bytes that need to be moved (the length of the item)
;	output: the item is right aligned, but still needs left end padding

move_item
	movff	POSTDEC0 , POSTDEC1
	dcfsnz	WREG
	return
	bra		move_item

left_pad
	movlw	'0'
left_pad_char
	movwf	POSTINC0
	dcfsnz	VTG_item_size
	return
	bra	left_pad_char	

	GLOBAL	serv_GPS, complete_GPS, set_gxx, display_GPS, display_XY, display_vtg, GPSInit , save_focus

	end	
