

#include "processor.inc"
#include "autop_defines.inc"

;********************************************************************************
;																				*
;	Math library for 16X16 signed and unsigned multiply, and					*
;	16X8 signed multiply.														*
;																				*
;	Arguments are ARG1L, ARG1H, ARG2L, ARG2H, for 16 bit values.				*
;	8 bit value is ARG2L, with ARG2H ignored.									*
;																				*
;	32 bit result is RES3:RES0.													*
;	24 bit result is RES2:RES0													*
;																				*
;	16X16 signed is MULS2X2														*
;	16X16 unsigned is MULU2X2													*
;																				*
;	16X8 signed is MULS2X1
;	16X8 unsigned is MULU2X1														*
;																				*
;	TRIG_LOOKUP																	*
;		computes the sine and cosine of THETA.									*
;																				*
;		THETA is the 1 byte angle, measured clockwise, 0 to 255.				*
;		256 is one full circle.													*
;																				*
;		256*sine and 256*cosine are returned as signed values in SINE and COSINE*
;		values are returned as 16 bits to handle the 2 extra bits needed.		*
;		SINEH:SINEL, and COSINEH:COSINEL.										*
;																				*
;		The 16 bit SINE and COSINE values can be multiplied by 16 bit values	*
;		using 16X16 signed multiply, producing a 16 bit result:					*
;																				*
;		RES3 can be thrown away, because it will be a sign extension,			*
;		RES0 can be thrown away as an insignificant fractional portion.			*
;																				*
;********************************************************************************


PROG1 	CODE

MULU2X2
	MOVF	ARG1L, W	; ARG1L*ARG2L->PRODH:PRODL
	MULWF	ARG2L	
	MOVFF	PRODH, RES1
	MOVFF	PRODL, RES0
	MOVF	ARG1H, W
	MULWF	ARG2H		; ARG1H*ARG2H->PRODH:PRODL
	MOVFF	PRODH, RES3
	MOVFF	PRODL, RES2
	MOVF	ARG1L, W
	MULWF	ARG2H		; ARG1L*ARG2H->PRODH:PRODL
	MOVF	PRODL, W
	ADDWF	RES1, F		; ADD CROSS PRODUCTS
	MOVF	PRODH, W
	ADDWFC	RES2, F
	CLRF	WREG
	ADDWFC	RES3, F
	MOVF	ARG1H, W
	MULWF	ARG2L		; ARG1H*ARG2L->PRODH:PRODL	
	MOVF	PRODL, W
	ADDWF	RES1, F		; ADD CROSS PRODUCTS
	MOVF	PRODH, W
	ADDWFC	RES2, F
	CLRF	WREG
	ADDWFC	RES3, F
	return

MULS2X2
	MOVF	ARG1L, W	; ARG1L*ARG2L->PRODH:PRODL
	MULWF	ARG2L	
	MOVFF	PRODH, RES1
	MOVFF	PRODL, RES0
	MOVF	ARG1H, W
	MULWF	ARG2H		; ARG1H*ARG2H->PRODH:PRODL
	MOVFF	PRODH, RES3
	MOVFF	PRODL, RES2
	MOVF	ARG1L, W
	MULWF	ARG2H		; ARG1L*ARG2H->PRODH:PRODL
	MOVF	PRODL, W
	ADDWF	RES1, F		; ADD CROSS PRODUCTS
	MOVF	PRODH, W
	ADDWFC	RES2, F
	CLRF	WREG
	ADDWFC	RES3, F
	MOVF	ARG1H, W
	MULWF	ARG2L		; ARG1H*ARG2L->PRODH:PRODL	
	MOVF	PRODL, W
	ADDWF	RES1, F		; ADD CROSS PRODUCTS
	MOVF	PRODH, W
	ADDWFC	RES2, F
	CLRF	WREG
	ADDWFC	RES3, F
	BTFSS	ARG2H, 7	; ARG2H:ARG2l NEG?
	BRA		SIGN_ARG12x2
	MOVF	ARG1L, W
	SUBWF	RES2
	MOVF	ARG1H, W
	SUBWFB	RES3
SIGN_ARG12x2
	BTFSS	ARG1H, 7	; ARG1H:ARG1L NEG?
	RETURN
	MOVF	ARG2L, W
	SUBWF	RES2
	MOVF	ARG2H, W
	SUBWFB	RES3
	return



MULS2X2ROT

;	slightly more efficient, used in ROTATE, because RES3, 0 are discarded anyway.
;	could possibly be made slightly more efficient by recognizing limits on cosine, sine values
;
	MOVF	ARG1L, W	; ARG1L*ARG2L->PRODH:PRODL
	MULWF	ARG2L	
	MOVFF	PRODH, RES1
;	MOVFF	PRODL, RES0
	MOVF	ARG1H, W
	MULWF	ARG2H		; ARG1H*ARG2H->PRODH:PRODL
;	MOVFF	PRODH, RES3
	MOVFF	PRODL, RES2
	MOVF	ARG1L, W
	MULWF	ARG2H		; ARG1L*ARG2H->PRODH:PRODL
	MOVF	PRODL, W
	ADDWF	RES1, F		; ADD CROSS PRODUCTS
	MOVF	PRODH, W
	ADDWFC	RES2, F
;	CLRF	WREG
;	ADDWFC	RES3, F
	MOVF	ARG1H, W
	MULWF	ARG2L		; ARG1H*ARG2L->PRODH:PRODL	
	MOVF	PRODL, W
	ADDWF	RES1, F		; ADD CROSS PRODUCTS
	MOVF	PRODH, W
	ADDWFC	RES2, F
;	CLRF	WREG
;	ADDWFC	RES3, F
	BTFSS	ARG2H, 7	; ARG2H:ARG2l NEG?
	BRA		SIGN_ARG12x2R
	MOVF	ARG1L, W
	SUBWF	RES2
;	MOVF	ARG1H, W
;	SUBWFB	RES3
SIGN_ARG12x2R
	BTFSS	ARG1H, 7	; ARG1H:ARG1L NEG?
	RETURN
	MOVF	ARG2L, W
	SUBWF	RES2
;	MOVF	ARG2H, W
;	SUBWFB	RES3
	return


MULU2X1
	MOVF	ARG1L, W	; ARG1L*ARG2L->PRODH:PRODL
	MULWF	ARG2L	
	MOVFF	PRODH, RES1
	MOVFF	PRODL, RES0
	CLRF	RES3
	CLRF	RES2
	MOVF	ARG1H, W
	MULWF	ARG2L		; ARG1H*ARG2L->PRODH:PRODL	
	MOVF	PRODL, W
	ADDWF	RES1, F		; ADD CROSS PRODUCTS
	MOVF	PRODH, W
	ADDWFC	RES2, F
	return

MULS2X1
	MOVF	ARG1L, W	; ARG1L*ARG2L->PRODH:PRODL
	MULWF	ARG2L	
	MOVFF	PRODH, RES1
	MOVFF	PRODL, RES0
	CLRF	RES3
	CLRF	RES2
	MOVF	ARG1H, W
	MULWF	ARG2L		; ARG1H*ARG2L->PRODH:PRODL	
	MOVF	PRODL, W
	ADDWF	RES1, F		; ADD CROSS PRODUCTS
	MOVF	PRODH, W
	ADDWFC	RES2, F
	BTFSS	ARG2L, 7	; ARG2L NEG?
	BRA		SIGN_ARG12x1
	MOVF	ARG1L, W
	SUBWF	RES1
	MOVF	ARG1H, W
	SUBWFB	RES2
SIGN_ARG12x1
	BTFSS	ARG1H, 7	; ARG1H:ARG1L NEG?
	RETURN
	MOVF	ARG2L, W
	SUBWF	RES2
	return



ROTATE

	rcall	TRIG_LOOKUP

	movff	COSINEH	, ARG1H
	movff	COSINEL , ARG1L

	movff	X_rectH , ARG2H
	movff	X_rectL , ARG2L

	rcall	MULS2X2ROT

	movff	RES2 , X_tempH
	movff	RES1 , X_tempL

	movff	SINEH , ARG1H
	movff	SINEL , ARG1L

	rcall	MULS2X2ROT

	movff	RES2 , Y_tempH
	movff	RES1 , Y_tempL

	movff	Y_rectH , ARG2H
	movff	Y_rectL , ARG2L

	rcall	MULS2X2ROT

	movf	RES1 , W
	subwf	X_tempL , F
	movf	RES2 , W
	subwfb	X_tempH , F

	movff	COSINEH	, ARG1H
	movff	COSINEL , ARG1L

	rcall	MULS2X2ROT

	movf	RES1 , W
	addwf	Y_tempL , F
	movf	RES2 , W
	addwfc	Y_tempH , F

	movff	X_tempH , X_rectH
	movff	X_tempL , X_rectL
	movff	Y_tempH , Y_rectH
	movff	Y_tempL , Y_rectL

	return


TRIG_LOOKUP					; THETA IS THE ANGLE, RESULT IN SINE, COSINE, H AND L
	rcall	sine_lookup
	rcall	cosine_lookup
	return

sine_lookup
	ld_tblptr	sine_table
	movf	THETA , W
	addwf	TBLPTRL , F
	movlw	0x0
	addwfc	TBLPTRH , F

	movf	THETA , W
	addwf	TBLPTRL , F
	movlw	0x0
	addwfc	TBLPTRH , F

	tblrd*+
	movff	TABLAT , SINEL
	tblrd*+
	movff	TABLAT , SINEH

	return

cosine_lookup
	ld_tblptr	sine_table

	movf	THETA , W
	addlw	0x40
	addwf	TBLPTRL , F
	movlw	0x0
	addwfc	TBLPTRH , F

	movf	THETA , W
	addlw	0x40
	addwf	TBLPTRL , F
	movlw	0x0
	addwfc	TBLPTRH , F

	tblrd*+
	movff	TABLAT , COSINEL
	tblrd*+
	movff	TABLAT , COSINEH

	return

	GLOBAL	MULS2X2 , MULU2X2 , MULU2X1, MULS2X1, TRIG_LOOKUP

	GLOBAL	ROTATE

rect_to_polar

	clrf	theta_temp
	movlw	B'01000000'
	movwf	delta_theta

cordic_step

	movff	delta_theta , THETA
	btfss	Y_rectH , 7
	negf	THETA
	rcall	ROTATE
	movf	THETA , W
	addwf	theta_temp , F
	rrncf	delta_theta
	btfss	delta_theta , 7
	bra		cordic_step
	btfss	Y_rectH , 7
	decf	theta_temp , F
	movff	theta_temp, THETA
	negf	THETA
	return
	
	GLOBAL	rect_to_polar

sine_table

	DW	.0
	DW	.6
	DW	.13
	DW	.19
	DW	.25
	DW	.31
	DW	.38
	DW	.44
	DW	.50
	DW	.56
	DW	.62
	DW	.68
	DW	.74
	DW	.80
	DW	.86
	DW	.92
	DW	.98
	DW	.104
	DW	.109
	DW	.115
	DW	.121
	DW	.126
	DW	.132
	DW	.137
	DW	.142
	DW	.147
	DW	.152
	DW	.157
	DW	.162
	DW	.167
	DW	.172
	DW	.177
	DW	.181
	DW	.185
	DW	.190
	DW	.194
	DW	.198
	DW	.202
	DW	.206
	DW	.209
	DW	.213
	DW	.216
	DW	.220
	DW	.223
	DW	.226
	DW	.229
	DW	.231
	DW	.234
	DW	.237
	DW	.239
	DW	.241
	DW	.243
	DW	.245
	DW	.247
	DW	.248
	DW	.250
	DW	.251
	DW	.252
	DW	.253
	DW	.254
	DW	.255
	DW	.255
	DW	.256
	DW	.256
	DW	.256
	DW	.256
	DW	.256
	DW	.255
	DW	.255
	DW	.254
	DW	.253
	DW	.252
	DW	.251
	DW	.250
	DW	.248
	DW	.247
	DW	.245
	DW	.243
	DW	.241
	DW	.239
	DW	.237
	DW	.234
	DW	.231
	DW	.229
	DW	.226
	DW	.223
	DW	.220
	DW	.216
	DW	.213
	DW	.209
	DW	.206
	DW	.202
	DW	.198
	DW	.194
	DW	.190
	DW	.185
	DW	.181
	DW	.177
	DW	.172
	DW	.167
	DW	.162
	DW	.157
	DW	.152
	DW	.147
	DW	.142
	DW	.137
	DW	.132
	DW	.126
	DW	.121
	DW	.115
	DW	.109
	DW	.104
	DW	.98
	DW	.92
	DW	.86
	DW	.80
	DW	.74
	DW	.68
	DW	.62
	DW	.56
	DW	.50
	DW	.44
	DW	.38
	DW	.31
	DW	.25
	DW	.19
	DW	.13
	DW	.6
	DW	.0
	DW	-.6
	DW	-.13
	DW	-.19
	DW	-.25
	DW	-.31
	DW	-.38
	DW	-.44
	DW	-.50
	DW	-.56
	DW	-.62
	DW	-.68
	DW	-.74
	DW	-.80
	DW	-.86
	DW	-.92
	DW	-.98
	DW	-.104
	DW	-.109
	DW	-.115
	DW	-.121
	DW	-.126
	DW	-.132
	DW	-.137
	DW	-.142
	DW	-.147
	DW	-.152
	DW	-.157
	DW	-.162
	DW	-.167
	DW	-.172
	DW	-.177
	DW	-.181
	DW	-.185
	DW	-.190
	DW	-.194
	DW	-.198
	DW	-.202
	DW	-.206
	DW	-.209
	DW	-.213
	DW	-.216
	DW	-.220
	DW	-.223
	DW	-.226
	DW	-.229
	DW	-.231
	DW	-.234
	DW	-.237
	DW	-.239
	DW	-.241
	DW	-.243
	DW	-.245
	DW	-.247
	DW	-.248
	DW	-.250
	DW	-.251
	DW	-.252
	DW	-.253
	DW	-.254
	DW	-.255
	DW	-.255
	DW	-.256
	DW	-.256
	DW	-.256
	DW	-.256
	DW	-.256
	DW	-.255
	DW	-.255
	DW	-.254
	DW	-.253
	DW	-.252
	DW	-.251
	DW	-.250
	DW	-.248
	DW	-.247
	DW	-.245
	DW	-.243
	DW	-.241
	DW	-.239
	DW	-.237
	DW	-.234
	DW	-.231
	DW	-.229
	DW	-.226
	DW	-.223
	DW	-.220
	DW	-.216
	DW	-.213
	DW	-.209
	DW	-.206
	DW	-.202
	DW	-.198
	DW	-.194
	DW	-.190
	DW	-.185
	DW	-.181
	DW	-.177
	DW	-.172
	DW	-.167
	DW	-.162
	DW	-.158
	DW	-.153
	DW	-.147
	DW	-.142
	DW	-.137
	DW	-.132
	DW	-.126
	DW	-.121
	DW	-.115
	DW	-.109
	DW	-.104
	DW	-.98
	DW	-.92
	DW	-.86
	DW	-.80
	DW	-.74
	DW	-.68
	DW	-.62
	DW	-.56
	DW	-.50
	DW	-.44
	DW	-.38
	DW	-.31
	DW	-.25
	DW	-.19
	DW	-.13
	DW	-.6
	DW	.0		;	not really needed, but it can't hurt

	end
