

#include "processor.inc"
#include "autop_defines.inc"

;********************************************************************************
;																				*
;	Steering module, executed 10 times per second to keep the vehicle			*
;	pointed in the desired direction, using a simple proportional control.		*
;	The servo angle is set proportional to the desired direction minus the		*
;	actual direction.															*
;																				*
;	All directions are measured as a 1 byte unsigned number, clockwise from		*
;	"north": 0 is N, 64 is E, 128 is S, and 192 is W.							*
;																				*
;	actualDir is updated from the compass 10 times per second.					*
;	desiredDir is determined by the navigation module from GPS info, once per	*
;	second. strngGain is the one byte steering gain, with an implied binary		*
;	point between the upper and lower 4 bits. In other words, a "engineering"	*
;	gain of one translates to a binary value of 0x10, or 16 decimal.			*
;	strngGain is an 8 bit signed number, so the feedback can be "flipped" if	*
;	need be. strngGain could also be set "on the fly".							*
;																				*
;********************************************************************************


PROG1 	CODE

SteerInit					

	movlw	steering_gain		;	initialize steering gain
	movwf	strngGain
	clrf	strngTrim			;	clear the steering trim
	clrf	speedTrim			;	clear the speed trim
	clrf	desiredSpeed		;	clear the desired speed
	return

Steer
	movf	desiredSpeed, W
	addwf	speedTrim, W
	bov		trim_sp_ov
	btg		WREG , 7
	movwf	PWM2_dc					;	Move the intermediate result to its final destination.
	bra		set_dir

trim_sp_ov
	btfss	WREG , 7
	bra		sp_clamp_min
	bra		sp_clamp_max

sp_clamp_max
	movlw	0xFF					;	otherwise, clamp to maximum, if overflow and not negative
	movwf	PWM2_dc					;	result is the PWM duty cycle
	bra		set_dir
sp_clamp_min
	movlw	0x00					;	clamp to minimum, result is PWM duty cycle
	movwf	PWM2_dc
	bra		set_dir


set_dir


;
;	for debugging purposes

;	movff	actualDir,	PWM1_dc					;	Move the intermediate result to its final destination.
;	return

;	end debugging

	movf	actualDir, W			;	actual direction in W
	subwf	desiredDir, W			;	subtracts actual (W) from desired (F), answer in W
	movwf	errorDir				;	save the signed difference
	smult	strngGain , errorDir	;	signed multiply, error times steering gain
									;   16 times the answer ends up in PRODH:PRODL
	movlw	0x08					;	add the equivalent of 1/2 msec pulse for straight ahead

	addwf	PRODH, F
	movf	PRODH, W				;	retrieve MSbyte of result
	andlw	0xF0					;	check for overflow
	bz		steer_normal			;	no "clamping" needed
	bn		steer_min				;	clamp to minimum for a "negative" time width pulse
	movlw	0xFF
	bra		str_trm_adjst

steer_min
	clrf	WREG
	bra		str_trm_adjst

steer_normal						;	unclamped case, must shift by 4 bits by using "swap:
	movlw	0xF0					;	mask for upper 4 bits
	andwf	PRODL, W				;	get the upper 4 bits of PRODL, put in W
	iorwf	PRODH, W				;	"or" the lower 4 bits of PRODH into W
									;	At this point the result is in W, except the nibbles are swapped.
	swapf	WREG					;	Swap the nibbles for the 4 bit shift.
;
;	here is where the trim comes in.
;	add the trim, test for overflow.
;
str_trm_adjst
	btg		WREG , 7
	addwf	strngTrim , W
	bov		trim_ov
	btg		WREG , 7
	movwf	PWM1_dc					;	Move the intermediate result to its final destination.
	return

trim_ov
	btfss	WREG , 7
	bra		str_clamp_min
	bra		str_clamp_max

str_clamp_max
	movlw	0xFF					;	otherwise, clamp to maximum, if overflow and not negative
	movwf	PWM1_dc					;	result is the PWM duty cycle
	return
str_clamp_min
	movlw	0x00					;	clamp to minimum, result is PWM duty cycle
	movwf	PWM1_dc
	return


	GLOBAL	Steer, SteerInit


	end
